//*****************************************************************************
//      Template Definitions for:
//		1, 2 and 3 dimension arrays.
//
//      Written by Stuart Prescott:
//              Written:                12/09/1996
//              Last Modified:          26/10/1996
//
//      System headers:
//              Streamed I/O
//
//*****************************************************************************
  
#ifndef SAFE_ARRAY_TEMPLATE_DEFINITIONS_H  
#define SAFE_ARRAY_TEMPLATE_DEFINITIONS_H 1  
#include <iostream.h>  

#define VERBOSEARRAYS 	0  
#define SAFEARRAYS 	0  

template <class T> class array1  
{
    private:  
	T* m;  
	int lb1, ub1;  
        int d1;  
  
    public:  
	//CONSTRUCTORS for initialisation  
        array1 (int a, int b)           //lb, ub  
	    {lb1=a; ub1=b; d1=ub1-lb1+1; m = new T[d1];  
	     if (m==0) cerr << "null ptr returned for array1\n";  
             #if VERBOSEARRAYS  
                cout <<"array1: this= " << this  
		     <<" m= "<< m <<"\n";  
             #endif  
             }  
        //DESTRUCTORS  
        ~array1 ()  
            {  
            #if VERBOSEARRAYS  
                cout << "deleting m1...";  
            #endif  
            delete[] m;  
            #if VERBOSEARRAYS  
                cout << "done\n";  
            #endif  
            }  
  
	//FUNCTIONS  
	//set elem  
	void set(int a, const T& v)  
            {  
            #if SAFEARRAYS  
                if ( (a>=lb1) && (a<=ub1) )  
                {  
                    m[a-lb1]=v;  
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("  %d  ", a);  
                    cerr<<"***********BAD array subscript:  " <<  a ;  
                    char k; cin >>k;  
                    //exit (-1);  
                }  
            #else  
                m[a-lb1]=v;  
            #endif  
            }  
  
        //get elem  
	T get(int a) const  
            {  
            #if SAFEARRAYS  
                if ( (a>=lb1)&&(a<=ub1) )  
                {  
                    return m[a-lb1];  
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("%d ", a);  
                    cerr<<"***********BAD array subscript:  " <<  a ;  
                    char k; cin >>k;  
                    //exit (-1);  
                    return 0;  
                }  
            #else  
                return m[a-lb1];  
            #endif  
            }  
  
        //get ub  
	int max() const  
	    {return ub1;}  
	int max(int dim) const  
	    {if (dim==1) {return ub1;} else {cerr<<"bad dimension"; return 0;} }  
  
        //get lb  
        int min() const  
	    {return lb1;}  
        int min(int dim) const  
	    {if (dim==1) {return lb1;} else {cerr<<"bad dimension"; return 0;} }  
  
        // allow streamed output  
        friend ostream& operator<< (ostream& s, const array1<T>& v)  
            {for (int c=v.lb1; c<=v.ub1; ++c) s << c <<": "<< v.get(c)<<" , ";  
             return s;}  
};  
  
typedef array1<double>  doublearray1;  
typedef array1<int>     intarray1;  
  
template <class T> class array2  
{  
    private:  
	T* m;  
	int lb1, ub1;  
        int d1 ;  
	int lb2, ub2;  
        int d2 ;  
  
    public:  
	//CONSTRUCTORS for initialisation  
	//lb, ub  
    	array2 (int a, int b, int c, int d)  
	    {lb1=a; ub1=b; d1=ub1-lb1+1;  
	     lb2=c; ub2=d; d2=ub2-lb2+1;  
	     m = new T[(d1*d2)];  
	     if (m==0) cerr << "null ptr returned for array2";  
             #if VERBOSEARRAYS  
                cout<<"array2. this= "<<this<<" m= "<<m<<"\n";  
             #endif  
             }  
	//DESTRUCTORS  
        ~array2 ()  
            {  
            #if VERBOSEARRAYS  
                cout << "deleting m2...";  
            #endif  
            delete[] m;  
            #if VERBOSEARRAYS  
                cout << "done\n";  
            #endif  
            }  
  
	//FUNCTIONS  
	//set elem  
	void set(int a, int b, const T& v)  
        {  
            #if SAFEARRAYS
                if ( (a>=lb1) && (a<=ub1) && (b>=lb2) && (b<=ub2) )  
                {  
                    m[(b-lb2)*d1 + a-lb1]= v;  
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("  %d  ", a);  
                    cerr<<"***********BAD array subscript:  " <<  a <<", "<<b;  
		    char k; cin >>k;  
                    //exit (-1);  
                }
	    #else
		m[(b-lb2)*d1 + a-lb1]= v;
	    #endif
        }  
  
        //get elem  
	T get(int a, int b) const  
	{  
            #if SAFEARRAYS            
                if ( (a>=lb1) && (a<=ub1) && (b>=lb2)&&(b<=ub2) )  
                {  
                    return m[(b-lb2)*d1 + a-lb1];  
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("%d ", a);  
                    cerr<<"***********BAD array subscript:  " <<  a <<", "<<b;  
		    char k; cin >>k;  
                    //exit (-1);  
                    return 0;  
                }  
	    #else
                return m[(b-lb2)*d1 + a-lb1];
	    #endif
        }  
  
        //get lb  
	int max(int dim) const  
	    {if ( (dim==1) || (dim==2) ) {return (dim==2 ? ub2 : ub1);}  
		else {cerr << "Bad dimension for 2d " << dim; return 0;}}  
  
        //get ub  
        int min(int dim) const  
	    {if ( (dim==1) || (dim==2) ) {return (dim==2 ? lb2 : lb1);}  
		else {cerr << "Bad dimension for 2d " << dim; return 0;}}  
  
        // allow streamed output  
        friend ostream& operator<< (ostream& s, const array2<T>& v)  
            {for (int c=0; c<=(v.d1*v.d2); ++c)  
		s << int(c/v.d1+v.lb1) << ", "  
                  << ((c%v.d1)+v.lb2) << "\t"  
		  << (v.m[c]) << "\n";  
             return s;}  
  
};  
typedef array2<double>  doublearray2;  
typedef array2<int>     intarray2;  
  
  
template <class T> class array3  
{  
    private:  
	T* m;  
	int lb1, ub1;  
        int d1 ;  
	int lb2, ub2;  
        int d2 ;  
	int lb3, ub3;  
        int d3 ;  
	int p;					    //the size of the plane  
  
    public:  
	//CONSTRUCTORS for initialisation  
	  //lb, ub  
    	array3 (int a, int b, int c, int d, int e, int f)  
	    {lb1=a; ub1=b; d1=ub1-lb1+1;  
	     lb2=c; ub2=d; d2=ub2-lb2+1;  
	     lb3=e; ub3=f; d3=ub3-lb3+1;  
	     p=d1*d2;  
	     m = new T[(p*d3)];  
	     if (m==0) cerr << "null ptr returned for array2";  
             #if VERBOSEARRAYS  
                cout<<"array3. this= "<<this<<" m= "<<m<<"\n";  
             #endif  
             }  
	//DESTRUCTORS  
        ~array3 ()  
            {  
            #if VERBOSEARRAYS  
                cout << "deleting m3...";  
            #endif  
            delete[] m;  
            #if VERBOSEARRAYS  
                cout << "done\n";  
            #endif  
            }  
  
	//FUNCTIONS  
	//set elem  
	void set(int a, int b, int c,const T& v)  
        {  
            #if SAFEARRAYS                        
                if ( (a>=lb1)&&(a<=ub1)&&(b>=lb2)&&(b<=ub2)&&(c>=lb3)&&(c<=ub3) )  
                {  
                    m[(c-lb3)*p + (b-lb2)*d1 + a-lb1] = v;  
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("  %d  ", a);  
                    cerr<<"***********BAD array subscript: "<<a<<", "<<b<<", "<<c;  
		    char k; cin >>k;  
                    //exit (-1);  
                }  
	    #else
                m[(c-lb3)*p + (b-lb2)*d1 + a-lb1] = v;
	    #endif
        }  
  
        //get elem  
	T get(int a, int b, int c) const  
	{  
            #if SAFEARRAYS                                    
                if ( (a>=lb1)&&(a<=ub1)&&(b>=lb2)&&(b<=ub2)&&(c>=lb3)&&(c<=ub3) )  
                {  
                    return m[(c-lb3)*p + (b-lb2)*d1 + a-lb1];   
                }  
                else  
                {  
                    //cerr<<"BAD array subscript:" << form ("%d ", a);  
                    cerr<<"***********BAD array subscript: "<<a<<", "<<b<<", "<<c;  
		    char k; cin >>k;  
                    //exit (-1);  
                    return 0;  
                }  
            #else
                return m[(c-lb3)*p + (b-lb2)*d1 + a-lb1];
            #endif        
        }  
  
        //get lb  
	int max(int dim) const  
	    {if ( (dim>=1)||(dim<=3) )  
		    {return (dim==3 ? ub3 : (dim==2 ? ub2 :ub1));}  
		else {cerr << "Bad dimension for 3d " << dim; return 0;}}  
  
        //get ub  
        int min(int dim) const  
	    {if ( (dim>=1)||(dim<=3) )   
		    {return (dim==3 ? lb3 : (dim==2 ? lb2 :lb1));}  
		else {cerr << "Bad dimension for 3d " << dim; return 0;}}  
  
        // allow streamed output  
        friend ostream& operator<< (ostream& s, const array3<T>& v)  
            {for (int c=0; c<=(v.p*v.d3); ++c)  
	         s << int(c%v.p+v.lb1)<<", "<<int((c%v.p)/v.d1+v.lb2) << ", "  
		   << ((c/v.p)+v.lb3) << "\t" << (v.m[c]) << "\n";  
             return s;}  
  
};  
typedef array3<double>  doublearray3;  
typedef array3<int>     intarray3;  
  
#undef VERBOSEARRAYS  
#undef SAFEARRAYS  
  
#endif  
