//*****************************************************************************
// Hydgrogen 3D:
//	Rendering 3D representaitons of atomic orbitals.
//
//	See the accompanying file "hyd_info.txt" 
//	    for more information
//
//      Written by Stuart Prescott:
//              Written:                12/08/1996
//              Last Modified:          26/10/1996
//
//      Libraries Required:
//              Athena Widget Set
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//
//      Other Object Classes used:
//              ArcBall
//		control panel
//              colours
//              fonts
//		callbacks
//		drawing objects
//
//*****************************************************************************

extern "C" {  
    #include <libsx.h>  
}  
  
#include <iostream.h>  
  
#include "main.h"	        // type defs for main data structure 
#include "callback.h"           // prototypes for callback functions  
#include "arcball.h"	        // prototypes for the arcball driver  
#include "objects.h"	        // prototypes for the h3d calculation engine  

#include "fontdefs.h"		// the fonts info
#include "coldefs.h"	        // the colour initialisations
  
extern ColourDefs *pal;
extern FontDefs *fonts;
  
//**************************************************************  
int  
main(int argc, char **argv)  
{  
	  
	MyProgram mydata;  
	  
	init_display(argc, argv, &mydata);  
  
	MainLoop();  

	//program should never reach here . . .
	exit(1);
}  
  
//**************************************************************  
void init_display(int argc, char **argv, MyProgram *me)  
{  
  
    // initialise display . . .  
    if (OpenDisplay(argc, argv) == FALSE)  
    {  
	cout << "Can't open display!\n";  
	return;  
    }  
  
    // set up the draw areas 
    me->mainwin.draw_area  = MakeDrawArea(640, 480, Redisplay, me);  
    me->mainwin.title_area = MakeDrawArea(150, 187, TitleRedisplay, 
					(me->mainwin.orbital.view));  
  
    // set up q to quit in draw_area  
    SetKeypressCB(me->mainwin.draw_area, Key_Quit_Func);  
 
    //*****start the arcball 
    me->arcballwin = new ArcballWin(TOP_LEVEL_FORM,  
				    PLACE_RIGHT, me->mainwin.draw_area,
				    NO_CARE, NULL, me);

    SetWidgetPos (me->mainwin.title_area, 
		  PLACE_UNDER, me->arcballwin->Where(), 
		  PLACE_RIGHT, me->mainwin.draw_area);

    //*****start the control panel		
    me->panelwin = new Panel(150, 131,
				TOP_LEVEL_FORM,
		 		PLACE_UNDER, me->mainwin.title_area, 
				PLACE_RIGHT, me->mainwin.draw_area, me);

    //*****set the necessary parameters for the orbital:
    // where to put the pics
    me->mainwin.orbital.SetDestination(me->mainwin.draw_area);
    // where to put the title
    me->mainwin.orbital.SetTitleLoc   (me->mainwin.title_area);
    // provide a memory interface between the arcball and orbital
    me->mainwin.orbital.AttachRotator (me->arcballwin->OutputLoc());
    // provide a memory interface between the selector and orbital
    me->mainwin.orbital.AttachSelector(me->mainwin.orbitalselect.OutputLoc()); 
    // provide an on/off switch for the pics (and title) 
    me->mainwin.orbital.AttachModeSwitch(&(me->mode));

    //******* Show display so that font and colour initialisations can be done.
    ShowDisplay();  

    //***** set up the colour palettes and the fonts 
    pal	    = new ColourDefs();
    fonts   = new FontDefs;  

    // initialise the orbital specific stuff.
    SetupOrbitals(me);

    return;
}  
