//*****************************************************************************
//      Header file for:
//		Drawing Objects: generic object base class and
//				 orbital
//
//      Written by Stuart Prescott:
//              Written:                01/09/1996
//              Last Modified:          26/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//		Strings
//
//      Other Object Classes used:
//              template arrays
//              3D points / vectors / matrices
//              main program data structure
//              orbital id and name
//              orbital drawing
//
//*****************************************************************************

#ifndef DRAWING_OBJECT_DEFINITIONS_H
#define DRAWING_OBJECT_DEFINITIONS_H 1

#include <string.h>

#include "arraytem.h"
#include "vectors.h"
#include "main.h"
#include "orbid.h"
#include "h3d.h"

#define title(X) ( ((X)==1) ? "WAVEFUNCTION" : "DENSITY" )

#ifndef ACTIONTYPE 
#define ACTIONTYPE 1
    enum actionflagtype {WAITING_FOR_SELECTION, DRAWING_ORBITAL_DENSITY};  
#endif

class DrawObject
{
    public:
	//The position in space
	vector v;

	// the transposition matrix
        matrix m;

	//the place to put it
	Widget draw_area;

};    //DrawObject

class Orbital : public DrawObject
{
    public:
	Widget title_area;
        class OrbitalView *view;
	//actionflagtype *choosemode;
        matrix *trans;
        class OrbitalDescriptor *id;
        int gcol;

        int fntype;

        Orbital()  {view = new OrbitalView(5);}
        ~Orbital() {delete view;}

        void Show(int itype, int x, int y)
            {view->Show(id, itype, trans, gcol, draw_area, title_area, x, y);
             return;}

        void Hide(){return;}			//unimplimented - no need

        void SetDestination(Widget w) {draw_area=w; return;}
        void AttachRotator(matrix *t) {trans=t; return;}
        void AttachSelector(OrbitalDescriptor *o) {id=o; return;}
	void AttachModeSwitch(actionflagtype *o) {view->choosemode = o; return;};
	void SetTitleLoc(Widget w) {title_area=w; return;}

	void newint(int i) {delete view; new OrbitalView(i);}

};    //Orbital

#endif

